<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

if (!defined('_PS_VERSION_'))
  exit;

require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkJobManager.php");
require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImport_product.php");
require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImport_manufacturer.php");
require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkImport_category.php");

class AdminPolarisLinkEditCategoryMappageController extends ModuleAdminController
{
    private $Link;

    public function __construct()
	{
		$this->context = Context::getContext();			
		$this->bootstrap = true;        
					
		parent::__construct();		
    }
    
    private function prependNames(&$category, $prependName = null)
    {
        $category["fullname"] = ($prependName ? $prependName . " >> " : ""). $category["name"];

        foreach ($category["childs"] as &$c)
            $this->prependNames($c, $category["fullname"]);
    }

    private function computePrestashopCategories(&$pol_categories, $category, $prependName = null)
    {
        if ($category["name"] == "")
            return null; // on ajoute pas cette branche
        
        if (isset($pol_categories[$category["id_category"]]))
            return null; // on ajoute pas cette branche
                    
        $newChilds = array();
        foreach ($category["childs"] as &$c)
        {
            $nc = $this->computePrestashopCategories($pol_categories, $c, $category["name"]);
            if ($nc)
                $newChilds[] = $nc;
        }
        $category["childs"] = $newChilds;        

        return $category;
    }
	
    private function renderCategory(&$categories, &$category, $level = 0)
    {
        $output = '<tr class="'.($level == 0 ? 'rootcat' : 'selectable').'">';
        $output .= '<td class="text-left">';
        $output .= str_replace(" ", '<span class="levelspace"></span>', str_pad("", $level, " ",  STR_PAD_LEFT)) . "&nbsp;";
        $output .= $category["name"];
        $output .= "</td>";

        $output .= '<td class="text-left">';
        $output .= "<ul>";
        foreach ($category["bindings"] as $c)
        {
            if (isset($categories[$c]))
                $output .= '<li class="'.($category["bd"] == $c ? 'cat-def' : '').'">'.$categories[$c]["fullname"].'</li>';
        }
        $output .= "</ul>";
        $output .= "</td>";

        $url = $this->context->link->getAdminLink('AdminPolarisLinkEditCategoryMappage')."&id_link=".$this->Link->LinkID.'&cfgCat='.$category["id_category"];
        $output .= '<td class="text-right">';
        if ($level > 0)
            $output .= '<div class="btn-group"><a class="btn btn-default" href="'.$url.'"><i class="icon-edit"></i> Modifier</a></div>';
        $output .= '</td>';

        $output .= "</tr>";

        foreach ($category["childs"] as &$child)
            $output .= $this->renderCategory($categories, $child, $level + 1);

        return $output;
    }

    private function setDefaultAssociation($id_link, $id_category, $default_id, $old_id)
    {
        PrestaShopLogger::addLog(sprintf("Définition catégorie par défaut %d => %d pour mappage catégorie %d (PolarisLink %d)", $old_id, $default_id, $id_category, $id_link), 1, 1, $this->module->displayName, 1, true);

        $db = Db::getInstance();
       
        // Modification des catégories par défaut ...
        $query = sprintf('SELECT DISTINCT id_product FROM '._DB_PREFIX_.'product WHERE 
                            id_product IN (SELECT id_product FROM '._DB_PREFIX_.'polarislink_products_group) AND
                            id_product IN (SELECT id_product FROM '._DB_PREFIX_.'category_product WHERE id_category = %d)', 
                    $id_category, $id);

        $results = $db->executeS($query, true, false);
        if ($results === false)
            throw new PrestaShopDatabaseException($db->getMsgError());

        foreach ($results as $r)
        {
            $p = new Product($r["id_product"]);
            if (!empty($p->id))
            {
                if ($p->id_category_default != $default_id)
                {
                    $p->id_category_default = $default_id;
                    $p->save();
                }
            }
        }

        if (!$db->execute(sprintf("UPDATE "._DB_PREFIX_."polarislink_categories_map SET is_default = 0 WHERE id_link = %d AND id_category = %d", 
                                    $id_link, $id_category)))
            throw new Exception("impossible de définir les associations par défaut !");
        if (!$db->execute(sprintf("UPDATE "._DB_PREFIX_."polarislink_categories_map SET is_default = 1 WHERE id_link = %d AND id_category = %d AND id_category_l = %d", 
                                    $id_link, $id_category, $default_id)))
            throw new Exception("impossible de définir les associations par défaut !");
    }

    private function addAssociation($id_link, $id_category, $id)
    {
        PrestaShopLogger::addLog(sprintf("Ajout mappage catégorie %d => %d (lien %d)", $id_category, $id, $id_link), 1, 1, $this->module->displayName, 1, true);

        $db = Db::getInstance();

        // Ajout des produits gérés par la passerelle à cette catégorie
        $query = sprintf('SELECT DISTINCT id_product FROM '._DB_PREFIX_.'product WHERE 
                            id_product IN (SELECT id_product FROM '._DB_PREFIX_.'polarislink_products_group) AND
                            id_product IN (SELECT id_product FROM '._DB_PREFIX_.'category_product WHERE id_category = %d) AND
                            id_product NOT IN (SELECT id_product FROM '._DB_PREFIX_.'category_product WHERE id_category = %d)', 
                    $id_category, $id);

        $results = $db->executeS($query, true, false);
        if ($results === false)
            throw new PrestaShopDatabaseException($db->getMsgError());            

        if (!$db->execute(sprintf("INSERT INTO "._DB_PREFIX_."polarislink_categories_map(id_link, id_category, id_category_l) VALUES (%d,%d,%d)", 
                                    $id_link, $id_category, $id)))
            throw new Exception("impossible de créer l'association !");

        $categoryImporter = new PolarisLinkImport_category();
        $categoryImporter->Init(new PolarisLinkJobManager($this->Link), $this->Link, null);

        // On associe les produits
        foreach ($results as $r)
        {
            $product = new Product($r["id_product"]);
            if (!empty($product->id))
            {
                $mappings_allowed = $categoryImporter->getMappingInfoForProduct($product);

                foreach ($categoryImporter->getMappingInfoForProduct($product) as &$mappings_allowed)
                {                    
                    if (in_array($id, $mappings_allowed))
                    {
                        $product->addToCategories(array($id));

                        // Recalcul catégorie par défaut ...
                        $product_default_category = $categoryImporter->computeDefaultCategoryForProduct($product);
                        if (!empty($product_default_category) && $product_default_category != $product->id_category_default)
                        {
                            $categoryImporter->Log($product, "Mise à jour catégorie par défaut suite à mappage #%s -> #%s", 
                                [
                                    $product->id_category_default,
                                    $product_default_category
                                ]);
                            $product->id_category_default = $product_default_category;
                            $product->save();
                        }

                        break;
                    }
                }
            }
        }
    }

    private function removeAssociation($id_link, $id_category, $id)
    {
        PrestaShopLogger::addLog(sprintf("Supression mappage catégorie %d => %d (lien %d)", $id_category, $id, $id_link), 1, 1, $this->module->displayName, 1, true);

        $db = Db::getInstance();

        // Retraits des produits gérés par la passerelle à cette catégorie
        // sauf ceux qui y sont lié à travers une autre association !!
        $query = sprintf('SELECT DISTINCT id_product FROM '._DB_PREFIX_.'product WHERE 
                            id_product IN (SELECT id_product FROM '._DB_PREFIX_.'polarislink_products_group) AND
                            id_product IN (SELECT id_product FROM '._DB_PREFIX_.'category_product WHERE id_category = %d) AND
                            id_product IN (SELECT id_product FROM '._DB_PREFIX_.'category_product WHERE id_category = %d) AND                            
                            id_product NOT IN (
                                SELECT id_product FROM '._DB_PREFIX_.'category_product WHERE id_category IN (
                                    SELECT id_category FROM '._DB_PREFIX_.'polarislink_categories_map WHERE id_category != %d AND id_category_l = %d
                                )
                            )                            
                            ', 
                    $id_category, $id, $id_category, $id);

        $results = $db->executeS($query, true, false);
        if ($results === false)
            throw new PrestaShopDatabaseException($db->getMsgError());
                    
        if (!$db->execute(sprintf("DELETE FROM "._DB_PREFIX_."polarislink_categories_map WHERE id_link = %d AND id_category = %d AND id_category_l = %d", 
                                    $id_link, $id_category, $id)))
            throw new Exception("impossible de supprimer l'association !");

        $categoryImporter = new PolarisLinkImport_category();
        $categoryImporter->Init(new PolarisLinkJobManager($this->Link), $this->Link, null);
    
        foreach ($results as $r)
        {
            $product = new Product($r["id_product"]);
            if (!empty($product->id))
                $product->deleteCategory($id);

            // Recalcul catégorie par défaut ...
            $product_default_category = $categoryImporter->computeDefaultCategoryForProduct($product);
            if (!empty($product_default_category) && $product_default_category != $product->id_category_default)
            {
                // Si la catégorie par défaut ne fait pas encore partie des catégories du produit, alors on 
                // corrige le tir
                $categoryImporter->Log($product, "Mise à jour catégorie par défaut suite à mappage #%s -> #%s", 
                    [
                        $product->id_category_default,
                        $product_default_category
                    ]);
                $product->id_category_default = $product_default_category;
                $product->save();
            }
        }
    }
	
	/* On procède à l'affichage */
	public function renderList()
	{		
		$output = '';
		
		try
		{            
            $link = null;
            foreach (PolarisLinkSite::loadConfig() as $cfg)
            {
                if (!isset($_REQUEST["id_link"]))
                    $_REQUEST["id_link"] = $cfg->LinkID;    // Si aucun lien précisé, on prend le lien par défaut ...
                    
                if ($cfg->LinkID == $_REQUEST["id_link"])
                {
                    $link = $cfg;
                    break;
                }
            }
            if (!$link)
                throw new Exception("Configuration non trouvée");

            $this->Link = $link;
			$db = Db::getInstance();

            // Modification ?
            $cfgCat = isset($_REQUEST["cfgCat"]) && !empty($_REQUEST["cfgCat"]) ? $_REQUEST["cfgCat"] : null;
            if ($cfgCat && isset($_REQUEST["map"]) && $_REQUEST["map"] == 1)
            {
                $cats = (isset($_REQUEST["categoryBox"]) && is_array($_REQUEST["categoryBox"])) ? $_REQUEST["categoryBox"] : [];

                $defaultCat = 0;
                if (isset($_REQUEST["default_cat"]) && $_REQUEST["default_cat"] != "")
                    $defaultCat = substr($_REQUEST["default_cat"], 1);
                
                // Pas de rajout en auto de la catégorie par défaut
                /*if ($defaultCat > 0 && !in_array($defaultCat, $cats))
                    $cats[] = $defaultCat;*/

                $category = new Category($cfgCat);
                if (empty($category->id))
                    throw new Exception("Catégorie ".$cfgCat." introuvable !");

                $currentDefaultCat = 0;
                $existingsAssoc = array();
                $results = $db->executeS(sprintf('SELECT id_category_l, is_default FROM `'._DB_PREFIX_.'polarislink_categories_map` WHERE id_link = %d AND id_category = %d', $cfg->LinkID, $category->id), true, false);
			    if ($results === false)
                    throw new PrestaShopDatabaseException($db->getMsgError());
                
                foreach ($results as $r)
                {
                    $existingsAssoc[$r["id_category_l"]] = false;
                    if ($r["is_default"])
                        $currentDefaultCat = $r["id_category_l"];
                }

                foreach ($cats as $c)
                {
                    if (!isset($existingsAssoc[$c]))
                    {
                        // L'association n'existe pas, il faut la créer ...
                        $this->addAssociation($this->Link->LinkID, $cfgCat, $c);
                        $existingsAssoc[$c] = true;
                    }

                    $existingsAssoc[$c] = true;
                }

                // Toutes les associations qui n'existent plus => poubelle
                foreach ($existingsAssoc as $c => $v)
                {
                    if (!$v)
                    {
                        // On supprime l'association                        
                        $this->removeAssociation($this->Link->LinkID, $cfgCat, $c);
                    }
                }

                if ($defaultCat && $currentDefaultCat != $defaultCat)
                    $this->setDefaultAssociation($this->Link->LinkID, $cfgCat, $defaultCat, $currentDefaultCat);

                $cfgCat = null;
            }

            // Chargement des catégories (TOUTES)

            // Nous devons charger les catégories (pour aller plus vite)
            // Et construire l'arbre des dépendances
            $categories = array();

            $results = $db->executeS('SELECT c.id_category, c.id_parent, cl.name
                        FROM `'._DB_PREFIX_.'category` c
                        '.Shop::addSqlAssociation('category', 'c').'
                        LEFT JOIN `'._DB_PREFIX_.'category_lang` cl ON (c.`id_category` = cl.`id_category`'.Shop::addSqlRestrictionOnLang('cl').' AND `id_lang` = '.$link->LangID.")", true, false);
			if ($results === false)
				throw new PrestaShopDatabaseException($db->getMsgError());
				
            foreach ($results as $category)
            {
                $category["childs"] = array();
                $category["bindings"] = array();
                $category["bd"] = array();
                $categories[$category["id_category"]] = $category;
            }

            $results = $db->executeS('SELECT id_category, id_category_l, is_default FROM `'._DB_PREFIX_.'polarislink_categories_map` WHERE id_link = '.$cfg->LinkID, true, false);
			if ($results === false)
                throw new PrestaShopDatabaseException($db->getMsgError());

            foreach ($results as $c)
            {
                if (isset($categories[$c["id_category"]]) && isset($categories[$c["id_category_l"]]))
                {
                    $categories[$c["id_category"]]["bindings"][] = $c["id_category_l"];
                    if ($c["is_default"])
                        $categories[$c["id_category"]]["bd"] = $c["id_category_l"];
                }
            }

            foreach ($categories as $key => $category)
                if ($category["id_parent"])
                    $categories[$category["id_parent"]]["childs"][] = &$categories[$key];

            $pol_categories = array();
            $pol_categories[$link->CategoryForRFS] = true;
            $pol_categories[$link->CategoryForCollection] = true;
            $pol_categories[$link->CategoryForSeason] = true;
            $pol_categories[$link->CategoryForTags] = true;
            $pol_categories[$link->CategoryForCategory1] = true;
            $pol_categories[$link->CategoryForCategory2] = true;
            $pol_categories[$link->CategoryForCategory3] = true;
            $pol_categories[$link->CategoryForCategory4] = true;

            // Calcul des catégories Prestashop
            $ps_categories = array();
            foreach ($categories as &$category)
            {
                if (!$category["id_parent"])
                {
                    $c = $this->computePrestashopCategories($pol_categories, $category);
                    if ($c)
                        $ps_categories[] = $c;

                    $this->prependNames($category);
                }
            }

            if ($cfgCat)
            {
                if (!isset($categories[$cfgCat]))
                    throw new Exception("Catégorie ".$cfgCat." introuvable !");
                
                $category = $categories[$cfgCat];
                $output .= '<h2>Configuration de la catégorie Polaris<br>'. $category["fullname"] .'</h2>';

                $output .= '<form method="POST" action="'.$this->context->link->getAdminLink('AdminPolarisLinkEditCategoryMappage')."&id_link=".$this->Link->LinkID.'&cfgCat='.$category["id_category"].'">';

                $GLOBALS["pp-catDefault"] = $categories[$cfgCat]["bd"];                

                $output .= '<input type="hidden" name="map" value="1">';
                $root = Category::getRootCategory();
                
                $tree = new HelperTreeCategories('categories-treeview', 'Associer cette catégorie aux catégories Prestashop suivantes' );
                $tree->setUseCheckBox(true)
                        ->setAttribute('is_category_filter', $root->id)
                        ->setRootCategory($root->id)
                        ->setSelectedCategories($category["bindings"])
                        ->setUseSearch(true);

                // On ne peut pas utiliser les catégories Polaris !!
                // $tree->setDisabledCategories(array_keys($pol_categories));

                $treeStr = $tree->render();

                $treeStr = preg_replace_callback('|<input type="checkbox" name="categoryBox\[\]" value="(\d+)" (.*)>|', 
                    function ($matches) {
                        $id = $matches[1];
                        return 
                            '<input type="radio" name="default_cat" value="d'.$id.'"'.($id == $GLOBALS["pp-catDefault"] ? ' checked' : '').'>'.
                            '<input type="checkbox" name="categoryBox[]" value="'.$id.'" '.$matches[2].'>';
                            $matches[1];
                    }, $treeStr);                

                $output .= $treeStr;
                $output .= '<style>#expand-all-categories-treeview { display: none; }</style>';
                $output .= '<style>#collapse-all-categories-treeview { display: none; }</style>';

                $output .= '<button type="submit">Valider</button>';

                $output .= '</form>';
            }
            else
            {
                $output .= '
                <style>
                    .levelspace {
                        margin-left: 1em;
                    }

                    .selectable:hover td {
                        background: #ddd !important;
                    }

                    .cat-def {
                        font-weight: bold;
                    }

                    .rootcat td {
                        font-weight: bold;
                        font-size: 150% !important;
                        border-top: 1px solid #a0d0eb !important;
                    }
                </style>
                ';
                $output .= "<h2>Résumé des mappages</h2>";
                $output .= '<table class="table"><thead><tr><th>Quand un produit est associé à la catégorie Polaris</th><th>Associer ce produit aux catégories Prestashop</th><th></th></tr></thead>';
                $output .= '<tbody>';
                
                if (isset($categories[$link->CategoryForRFS]))
                    $output .= $this->renderCategory($categories, $categories[$link->CategoryForRFS]);

                if (isset($categories[$link->CategoryForCategory1]))
                    $output .= $this->renderCategory($categories, $categories[$link->CategoryForCategory1]);
                
                if (isset($categories[$link->CategoryForCategory2]))
                    $output .= $this->renderCategory($categories, $categories[$link->CategoryForCategory2]);
                
                if (isset($categories[$link->CategoryForCategory3]))
                    $output .= $this->renderCategory($categories, $categories[$link->CategoryForCategory3]);
                
                if (isset($categories[$link->CategoryForCategory4]))
                    $output .= $this->renderCategory($categories, $categories[$link->CategoryForCategory4]);
                
                if (isset($categories[$link->CategoryForCollection]))
                    $output .= $this->renderCategory($categories, $categories[$link->CategoryForCollection]);

                if (isset($categories[$link->CategoryForSeason]))
                    $output .= $this->renderCategory($categories, $categories[$link->CategoryForSeason]);

                $output .= '</tbody>';
                $output .= '</table>';
            }
		}
		catch (Exception $ex)
		{
			//PrestaShopLogger::addLog("Erreur lors de la conversion du site IPS : ".$ex->getMessage(), 2, 1, $this->module->displayName, 1, true);
			$output .= '<div class="alert alert-danger">'.Tools::displayError($ex->getMessage()).'</div>';
		}
		
		return $output;
	}
}